################################################################################################################################
#
# Description:	This constraint file applies to hardware features that are common across the entire SigFPGA product family.
#
# Rev History: 	07-14-2015:	Added the PCI Express clock and reset since they are not included in the endpoint core IP.
# 				10-13-2014:	Removed pin contraints for EMCCLK to work around a bug in Vivado 2014.2 and 2014.3.
# 				08-20-2014:	Inital release.
#
################################################################################################################################

###################################################### Timing Constraints ######################################################

# Set the 200 MHz external clock oscillator frequency.
create_clock -name clk200 -period 5 [get_ports clk200_p]
set_clock_groups -name async_clk200 -asynchronous -group [get_clocks clk200]

# Set the PCI Express system clock frequency to 100 MHz.
create_clock -name pci_sys_clk -period 10 [get_ports pci_sys_clk_p]

# Set false paths for all asynchronous reset signals, both hardware and software generated.
set_false_path -through [get_nets -hierarchical user_reset]
set_false_path -through [get_nets -hierarchical software_reset*]

###################################################### Pinout Constraints ######################################################
# Note: The external component and pin name connected to each PACKAGE_PIN is shown as a comment (#component:pin).
# Note: Pins attached to a connector through a buffer include both components separated by a "/".

# PCI Express reset and system clock pin assignments.
set_property PACKAGE_PIN D19 [get_ports sys_reset_n];  #PCI_EDGE:PERSTN
set_property PULLUP true [get_ports sys_reset_n]
set_property PACKAGE_PIN K6 [get_ports pci_sys_clk_p];  #PCI_EDGE:PCIREFCLKP
set_property PACKAGE_PIN K5 [get_ports pci_sys_clk_n];  #PCI_EDGE:PCIREFCLKN

# 200 MHZ clock pin assignments.
set_property PACKAGE_PIN E18 [get_ports clk200_p];  #TP53A:CLKP

# Serial bus pin assignments.
set_property PACKAGE_PIN E20 [get_ports i2c_scl];  #Connects to multiple components.
set_property PACKAGE_PIN F19 [get_ports i2c_sda];  #Connects to multiple components.
set_property PACKAGE_PIN A18 [get_ports spi_clk];  #Connects to multiple components.
set_property PACKAGE_PIN C19 [get_ports spi_csb_cmd[0]];  #74LVC595A:SHCP
set_property PACKAGE_PIN C17 [get_ports spi_csb_cmd[1]];  #74LVC595A:DS
set_property PACKAGE_PIN C18 [get_ports spi_csb_cmd[2]];  #74LVC595A:STCP
set_property PACKAGE_PIN A19 [get_ports spi_sdi];  #Connects to multiple components.
set_property PACKAGE_PIN B19 [get_ports spi_sdo];  #Connects to multiple components.
set_property PACKAGE_PIN L19 [get_ports synth_cmd[0]];  #SYNTHESIZER:CLOCK
set_property PACKAGE_PIN M16 [get_ports synth_cmd[1]];  #SYNTHESIZER:DATA
set_property PACKAGE_PIN K15 [get_ports synth_cmd[2]];  #SYNTHESIZER:ENABLE

# LED pin assignments.
set_property PACKAGE_PIN A17 [get_ports led_on[0]];  #LED:YELLOW
set_property PACKAGE_PIN B17 [get_ports led_on[1]];  #LED:GREEN

# Trigger pin assignments.
set_property PACKAGE_PIN G24 [get_ports coax_trig];  #74LVC1G17:Y/SMA
set_property PACKAGE_PIN F24 [get_ports gpio_trig];  #74LVC1G17:Y/A28100-15:Pin7

# Status pin assignments.
set_property PACKAGE_PIN F20 [get_ports alert_n];  #INA3221:CRIT&WARN and FXL6408:INTN
set_property PACKAGE_PIN N16 [get_ports fan_clear];  #74LVC1G17:A/TC670:CLEAR
set_property PACKAGE_PIN U16 [get_ports fan_status];  #TC670:ALERTN

# GPIO pin assignments.
set_property PACKAGE_PIN D23 [get_ports gpio[1]];  #FXL2TD245:A0/A28100-15:Pin1
set_property PACKAGE_PIN D24 [get_ports gpio[2]];  #FXL2TD245:A1/A28100-15:Pin2
set_property PACKAGE_PIN E25 [get_ports gpio[3]];  #FXL2TD245:A0/A28100-15:Pin3
set_property PACKAGE_PIN D25 [get_ports gpio[4]];  #FXL2TD245:A1/A28100-15:Pin4
set_property PACKAGE_PIN F22 [get_ports gpio[5]];  #FXL2TD245:A0/A28100-15:Pin5
set_property PACKAGE_PIN E23 [get_ports gpio[6]];  #FXL2TD245:A1/A28100-15:Pin6
set_property PACKAGE_PIN G25 [get_ports gpio_tristate];  #FXL2TD245:OEN

# User pin assignments.
set_property PACKAGE_PIN H11 [get_ports user[1]];  #5787082-7:Pin1
set_property PACKAGE_PIN H12 [get_ports user[2]];  #5787082-7:Pin35
set_property PACKAGE_PIN J10 [get_ports user[3]];  #5787082-7:Pin2
set_property PACKAGE_PIN J11 [get_ports user[4]];  #5787082-7:Pin36
set_property PACKAGE_PIN H13 [get_ports user[5]];  #5787082-7:Pin4
set_property PACKAGE_PIN J13 [get_ports user[6]];  #5787082-7:Pin38
set_property PACKAGE_PIN H8 [get_ports user[7]];  #5787082-7:Pin5
set_property PACKAGE_PIN H9 [get_ports user[8]];  #5787082-7:Pin39
set_property PACKAGE_PIN G14 [get_ports user[9]];  #5787082-7:Pin6
set_property PACKAGE_PIN H14 [get_ports user[10]];  #5787082-7:Pin40
set_property PACKAGE_PIN F10 [get_ports user[11]];  #5787082-7:Pin7
set_property PACKAGE_PIN G11 [get_ports user[12]];  #5787082-7:Pin41
set_property PACKAGE_PIN G9 [get_ports user[13]];  #5787082-7:Pin8
set_property PACKAGE_PIN G10 [get_ports user[14]];  #5787082-7:Pin42
set_property PACKAGE_PIN F8 [get_ports user[15]];  #5787082-7:Pin10
set_property PACKAGE_PIN F9 [get_ports user[16]];  #5787082-7:Pin44
set_property PACKAGE_PIN F12 [get_ports user[17]];  #5787082-7:Pin11
set_property PACKAGE_PIN G12 [get_ports user[18]];  #5787082-7:Pin45
set_property PACKAGE_PIN F13 [get_ports user[19]];  #5787082-7:Pin12
set_property PACKAGE_PIN F14 [get_ports user[20]];  #5787082-7:Pin46
set_property PACKAGE_PIN D11 [get_ports user[21]];  #5787082-7:Pin13
set_property PACKAGE_PIN E11 [get_ports user[22]];  #5787082-7:Pin47
set_property PACKAGE_PIN D8 [get_ports user[23]];  #5787082-7:Pin14
set_property PACKAGE_PIN D9 [get_ports user[24]];  #5787082-7:Pin48
set_property PACKAGE_PIN D10 [get_ports user[25]];  #5787082-7:Pin16
set_property PACKAGE_PIN E10 [get_ports user[26]];  #5787082-7:Pin50
set_property PACKAGE_PIN E12 [get_ports user[27]];  #5787082-7:Pin17
set_property PACKAGE_PIN E13 [get_ports user[28]];  #5787082-7:Pin51
set_property PACKAGE_PIN D13 [get_ports user[29]];  #5787082-7:Pin18
set_property PACKAGE_PIN D14 [get_ports user[30]];  #5787082-7:Pin52
set_property PACKAGE_PIN C11 [get_ports user[31]];  #5787082-7:Pin19
set_property PACKAGE_PIN C12 [get_ports user[32]];  #5787082-7:Pin53
set_property PACKAGE_PIN B9 [get_ports user[33]];  #5787082-7:Pin21
set_property PACKAGE_PIN C9 [get_ports user[34]];  #5787082-7:Pin55
set_property PACKAGE_PIN A10 [get_ports user[35]];  #5787082-7:Pin22
set_property PACKAGE_PIN B10 [get_ports user[36]];  #5787082-7:Pin56
set_property PACKAGE_PIN A8 [get_ports user[37]];  #5787082-7:Pin23
set_property PACKAGE_PIN A9 [get_ports user[38]];  #5787082-7:Pin57
set_property PACKAGE_PIN C13 [get_ports user[39]];  #5787082-7:Pin24
set_property PACKAGE_PIN C14 [get_ports user[40]];  #5787082-7:Pin58
set_property PACKAGE_PIN B11 [get_ports user[41]];  #5787082-7:Pin25
set_property PACKAGE_PIN B12 [get_ports user[42]];  #5787082-7:Pin59
set_property PACKAGE_PIN A12 [get_ports user[43]];  #5787082-7:Pin27
set_property PACKAGE_PIN A13 [get_ports user[44]];  #5787082-7:Pin61
set_property PACKAGE_PIN A14 [get_ports user[45]];  #5787082-7:Pin28
set_property PACKAGE_PIN B14 [get_ports user[46]];  #5787082-7:Pin62
set_property PACKAGE_PIN A15 [get_ports user[47]];  #5787082-7:Pin29
set_property PACKAGE_PIN B15 [get_ports user[48]];  #5787082-7:Pin63
set_property PACKAGE_PIN J14 [get_ports user[49]];  #5787082-7:Pin30
set_property PACKAGE_PIN J8 [get_ports user[50]];  #5787082-7:Pin64
set_property PACKAGE_PIN C16 [get_ports user[51]];  #FXL4TD245:A3/5787082-7:Pin31
set_property PACKAGE_PIN B16 [get_ports user[52]];  #FXL4TD245:A2/5787082-7:Pin65
set_property PACKAGE_PIN D15 [get_ports user[53]];  #FXL4TD245:A1/5787082-7:Pin33
set_property PACKAGE_PIN D16 [get_ports user[54]];  #FXL4TD245:A0/5787082-7:Pin67
set_property PACKAGE_PIN E15 [get_ports user[55]];  #FXL4TD245:A3/5787082-7:Pin34
set_property PACKAGE_PIN E16 [get_ports user[56]];  #FXL4TD245:A2/5787082-7:Pin68
set_property PACKAGE_PIN G17 [get_ports user[57]];  #FXL4TD245:A1/5787082-7:Pin37
set_property PACKAGE_PIN F18 [get_ports user[58]];  #FXL4TD245:A0/5787082-7:Pin9
set_property PACKAGE_PIN G15 [get_ports user[59]];  #FXL4TD245:A3/5787082-7:Pin49
set_property PACKAGE_PIN F15 [get_ports user[60]];  #FXL4TD245:A2/5787082-7:Pin20
set_property PACKAGE_PIN H16 [get_ports user[61]];  #FXL4TD245:A1/5787082-7:Pin60
set_property PACKAGE_PIN G16 [get_ports user[62]];  #FXL4TD245:A0/5787082-7:Pin32
set_property PACKAGE_PIN G19 [get_ports user_tristate];  #FXL4TD245:OEN

# GPIO/user direction port expander pin assignments.
set_property PACKAGE_PIN J21 [get_ports gpio_user_cmd[0]];  #74LVC595A:SHCP
set_property PACKAGE_PIN L23 [get_ports gpio_user_cmd[1]];  #74LVC595A:DS
set_property PACKAGE_PIN K21 [get_ports gpio_user_cmd[2]];  #74LVC595A:STCP

# BPI configuration flash pin assignments.
set_property PACKAGE_PIN J23 [get_ports bpi_a[0]];  #MT28GU512:A1
set_property PACKAGE_PIN K23 [get_ports bpi_a[1]];  #MT28GU512:A2
set_property PACKAGE_PIN K22 [get_ports bpi_a[2]];  #MT28GU512:A3
set_property PACKAGE_PIN L22 [get_ports bpi_a[3]];  #MT28GU512:A4
set_property PACKAGE_PIN J25 [get_ports bpi_a[4]];  #MT28GU512:A5
set_property PACKAGE_PIN J24 [get_ports bpi_a[5]];  #MT28GU512:A6
set_property PACKAGE_PIN H22 [get_ports bpi_a[6]];  #MT28GU512:A7
set_property PACKAGE_PIN H24 [get_ports bpi_a[7]];  #MT28GU512:A8
set_property PACKAGE_PIN H23 [get_ports bpi_a[8]];  #MT28GU512:A9
set_property PACKAGE_PIN G21 [get_ports bpi_a[9]];  #MT28GU512:A10
set_property PACKAGE_PIN H21 [get_ports bpi_a[10]];  #MT28GU512:A11
set_property PACKAGE_PIN H26 [get_ports bpi_a[11]];  #MT28GU512:A12
set_property PACKAGE_PIN J26 [get_ports bpi_a[12]];  #MT28GU512:A13
set_property PACKAGE_PIN E26 [get_ports bpi_a[13]];  #MT28GU512:A14
set_property PACKAGE_PIN F25 [get_ports bpi_a[14]];  #MT28GU512:A15
set_property PACKAGE_PIN G26 [get_ports bpi_a[15]];  #MT28GU512:A16
set_property PACKAGE_PIN K17 [get_ports bpi_a[16]];  #MT28GU512:A17
set_property PACKAGE_PIN K16 [get_ports bpi_a[17]];  #MT28GU512:A18
set_property PACKAGE_PIN L20 [get_ports bpi_a[18]];  #MT28GU512:A19
set_property PACKAGE_PIN J19 [get_ports bpi_a[19]];  #MT28GU512:A20
set_property PACKAGE_PIN J18 [get_ports bpi_a[20]];  #MT28GU512:A21
set_property PACKAGE_PIN J20 [get_ports bpi_a[21]];  #MT28GU512:A22
set_property PACKAGE_PIN K20 [get_ports bpi_a[22]];  #MT28GU512:A23
set_property PACKAGE_PIN K18 [get_ports bpi_rs[0]];  #MT28GU512:A24
set_property PACKAGE_PIN L17 [get_ports bpi_rs[1]];  #MT28GU512:A25
set_property PACKAGE_PIN B24 [get_ports bpi_d[0]];  #MT28GU512:DQ0
set_property PACKAGE_PIN A25 [get_ports bpi_d[1]];  #MT28GU512:DQ1
set_property PACKAGE_PIN B22 [get_ports bpi_d[2]];  #MT28GU512:DQ2
set_property PACKAGE_PIN A22 [get_ports bpi_d[3]];  #MT28GU512:DQ3
set_property PACKAGE_PIN A23 [get_ports bpi_d[4]];  #MT28GU512:DQ4
set_property PACKAGE_PIN A24 [get_ports bpi_d[5]];  #MT28GU512:DQ5
set_property PACKAGE_PIN D26 [get_ports bpi_d[6]];  #MT28GU512:DQ6
set_property PACKAGE_PIN C26 [get_ports bpi_d[7]];  #MT28GU512:DQ7
set_property PACKAGE_PIN C24 [get_ports bpi_d[8]];  #MT28GU512:DQ8
set_property PACKAGE_PIN D21 [get_ports bpi_d[9]];  #MT28GU512:DQ9
set_property PACKAGE_PIN C22 [get_ports bpi_d[10]];  #MT28GU512:DQ10
set_property PACKAGE_PIN B20 [get_ports bpi_d[11]];  #MT28GU512:DQ11
set_property PACKAGE_PIN A20 [get_ports bpi_d[12]];  #MT28GU512:DQ12
set_property PACKAGE_PIN E22 [get_ports bpi_d[13]];  #MT28GU512:DQ13
set_property PACKAGE_PIN C21 [get_ports bpi_d[14]];  #MT28GU512:DQ14
set_property PACKAGE_PIN B21 [get_ports bpi_d[15]];  #MT28GU512:DQ15
set_property PACKAGE_PIN D20 [get_ports bpi_adv_b];  #MT28GU512:ADVN
set_property PACKAGE_PIN C23 [get_ports bpi_fcs_b];  #MT28GU512:CEN
set_property PACKAGE_PIN L18 [get_ports bpi_fwe_b];  #MT28GU512:WEN
set_property PACKAGE_PIN M17 [get_ports bpi_foe_b];  #MT28GU512:OEN
set_property PACKAGE_PIN E21 [get_ports bpi_vpp];  #MT28GU512:VPP

################################################ Bitstream Settings ################################################

set_property BITSTREAM.CONFIG.BPI_SYNC_MODE Type1 [current_design]
set_property BITSTREAM.CONFIG.EXTMASTERCCLK_EN div-1 [current_design]

set_property CFGBVS GND [current_design]
set_property CONFIG_VOLTAGE 1.8 [current_design]
