--*******************************************************************************************************************************
--
-- Description: This model demonstrates the operation of the QDR II+ SRAM by performing a test at every available address.
--				A unique value is written to each location and then read back to verify that it matches the expected value.
--				The total number of errors detected are recorded in a register that can be accessed through software.
--					
--				Note:	The 200 MHz clock domain was selected for convenience to interface with the SRAM read/write ports,
--						but any clock domain can be used to control the interface.
--					
-- Rev History: 08-04-2016:	Changed the name of qdriip_sys_clk to qdriip_clk to distinguish spread spectrum clock name.
-- 				07-14-2015:	Initial release.
--
--*******************************************************************************************************************************

library ieee;
use ieee.std_logic_1164.all;
use IEEE.numeric_std.all; 

Library UNISIM;
use UNISIM.vcomponents.all;
library UNIMACRO;
use unimacro.Vcomponents.all;

use work.model_specific_package.all;
use work.transceiver_package.all;

entity qdr_demo is
	port (
	--  Reset and clocks supplied by the application logic.
	rst         		: in std_logic;							-- Active high asynchronous reset.
	clk 				: in std_logic;							-- Bus clock.
	clk200 				: in std_logic;							-- 200 MHz clock on global clock buffer.
	qdriip_clk			: in std_logic;							-- QDR II+ fixed frequency or spread spectrum clock.
	
	-- QDR II+ SRAM controller interface signals.
	sram_start			: in std_logic;							-- Single period start command to initiate the demonstration.
	sram_busy			: out std_logic;						-- Logic one indicates that the demonstration is active.
	sram_error			: out std_logic_vector(31 downto 0);	-- Error count from the last demonstration cycle.
	
	-- Serial clock lock command signal supplied from/to the application logic.
	dll_rst_start		: in std_logic; 						-- Connect to the ready output of the IDELAY CONTROLLER.
	clk_lck_status		: out std_logic; 						-- Logic one indicates that clocks are locked.
	
	--	The following signals connect to SRAM devices through FPGA pins.
	qdriip_cq			: in  std_logic;						-- QDR II+ SRAM echo clock.
	qdriip_q			: in  std_logic_vector(17 downto 0);	-- QDR II+ SRAM synchronous data outputs.
	qdriip_qvld			: in  std_logic;						-- QDR II+ SRAM read data valid flag.
	qdriip_k			: out std_logic;						-- QDR II+ SRAM input clock.
	qdriip_d			: out std_logic_vector(17 downto 0);	-- QDR II+ SRAM synchronous data inputs.
	qdriip_sa			: out std_logic_vector(QDR_ADDR_WIDTH-1 downto 0);	-- QDR II+ SRAM synchronous address inputs.
	qdriip_w_n			: out std_logic;						-- QDR II+ SRAM synchronous active low write enable.
	qdriip_r_n			: out std_logic;						-- QDR II+ SRAM synchronous active low read enable.
	qdriip_bw_n			: out std_logic_vector(1 downto 0);		-- QDR II+ SRAM synchronous byte enables.
	qdriip_dll_off_n	: out std_logic							-- QDR II+ SRAM active low DLL disable.
	);	
end qdr_demo;

architecture behavioral of qdr_demo is

	type QDR_DEMO_STATE is (idle, write, read, compare);	-- Type definitions for the QDR II+ state machine.
	signal demo_state 	: QDR_DEMO_STATE;					-- Current state.
	signal cnt_clr		: std_logic;						-- Logic one clears the address and expected data counters.
	signal start		: std_logic;						-- Logic one starts a demonstration cycle.
	signal wr_rdy		: std_logic;						-- Logic one indicates that the write port is ready to accept another command.
	signal wr_cmd		: std_logic;						-- Logic one commits a write cycle to the SRAM using the current address and data.
	signal wr_active	: std_logic;						-- Logic one begins the cycle of writing a value to every available address.
	signal wr_add_ce	: std_logic;						-- Logic one enables the write address counter.
	signal wr_add		: std_logic_vector(31 downto 0);	-- Address input to the write port.
	signal wr_data		: std_logic_vector(71 downto 0);	-- Data input to the write port.
	signal rd_rdy		: std_logic;						-- Logic one indicates that the read port is ready to accept another command.
	signal rd_cmd		: std_logic;						-- Logic one commits a read cycle to the SRAM using the current address.
	signal rd_active	: std_logic;						-- Logic one begins the cycle of reading the value from every available address.
	signal rd_add_ce	: std_logic;						-- Logic one enables the read address counter.
	signal rd_add		: std_logic_vector(31 downto 0);	-- Address input to the read port.
	signal rd_data		: std_logic_vector(71 downto 0);	-- Data output from the read port.
	signal rd_vld		: std_logic;						-- Logic one indicates that data is available from the requested read operation.
	signal rd_expect_ce	: std_logic;						-- Logic one enables the expected value address counter.
	signal rd_expect	: std_logic_vector(31 downto 0);	-- Count used to generate the expected read value.
	signal expected		: std_logic_vector(71 downto 0);	-- Expected value from each read transaction.
	signal error_clr	: std_logic;						-- Logic one clears the error counter.
	signal error_ce		: std_logic;						-- Logic one enables the error counter.
	signal error_detect	: std_logic;						-- Flag to indicate that an error has been detected.
	
begin

--*************************************************** INSTANTIATE COUNTERS *****************************************************

	-- Counter to generate write addresses that are also used to create the data field.
	WR_ADD_COUNTER : COUNTER_LOAD_MACRO
		generic map (DEVICE => "7SERIES", COUNT_BY => x"000000000001", WIDTH_DATA => 32)
		port map (Q => wr_add, CLK => clk200, CE => wr_add_ce, DIRECTION => '1', LOAD => '0', LOAD_DATA => (others => '0'), RST => cnt_clr);
		
	-- Counter to generate read addresses.
	RD_ADD_COUNTER : COUNTER_LOAD_MACRO
		generic map (DEVICE => "7SERIES", COUNT_BY => x"000000000001", WIDTH_DATA => 32)
		port map (Q => rd_add, CLK => clk200, CE => rd_add_ce, DIRECTION => '1', LOAD => '0', LOAD_DATA => (others => '0'), RST => cnt_clr);
		
	-- Counter that supplies the address needed to reproduce the expected result from a read transaction.
	RD_EXPECT_COUNTER : COUNTER_LOAD_MACRO
		generic map (DEVICE => "7SERIES", COUNT_BY => x"000000000001", WIDTH_DATA => 32)
		port map (Q => rd_expect, CLK => clk200, CE => rd_expect_ce, DIRECTION => '1', LOAD => '0', LOAD_DATA => (others => '0'), RST => cnt_clr);
		
	-- Counter to track the number of errors detected.
	QDR_ERR_COUNTER : COUNTER_LOAD_MACRO
		generic map (DEVICE => "7SERIES", COUNT_BY => x"000000000001", WIDTH_DATA => 32)
		port map (Q => sram_error, CLK => clk200, CE => error_ce, DIRECTION => '1', LOAD => '0', LOAD_DATA => (others => '0'), RST => error_clr);
			
--************************************************ STATE MACHINE TRANSITIONS ***************************************************

	-- Generate a single period start strobe in the 200 MHz clock domain.
	QDR_START_XING : entity work.clock_crossing
	port map (
		source_clk			=> clk,
		target_clk  		=> clk200,
		rst					=> rst,
		source_to_target	=> sram_start,
		target_enable		=> start
	);

	process (clk200,rst)
	begin
		if rst='1' then
		
			demo_state <= idle;

		elsif rising_edge(clk200) then
		
			case demo_state is
			
				-- Remain idle until commanded to start.
				when idle =>
					-- Wait for the start command.
					if start = '1' then
						demo_state <= write;
					end if;
					
				-- Initiate a cycle of write transactions.
				when write =>
					-- Wait for the write transactions to get two addresses ahead of the read transactions.
					if  wr_add(1) = '1' then
						demo_state <= read;
					end if;
					
				-- Initiate a cycle of read transactions.
				when read =>
					demo_state <= compare;
					
				-- Compare the read return values to the values written.
				when compare =>
					-- Wait for the last comparison to complete.
					if  rd_expect(QDR_ADDR_WIDTH) = '1' then
						demo_state <= idle;
					end if;
					
				when others  =>
					null;
			end case;
		end if;
	end process;
	
--*************************************************** STATE MACHINE EVENTS *****************************************************
	
	process (clk200,rst)
	begin
		if rst='1' then
		
			cnt_clr <= '1';
			error_clr <= '1';
			wr_active <= '0';
			rd_active <= '0';
			error_detect <= '0';

		elsif rising_edge(clk200) then
		
			-- Clear the address and expected data counters.
			if demo_state = idle then
				cnt_clr <= '1';
			else
				cnt_clr <= '0';
			end if;
		
			-- Clear the error counter.
			if demo_state = write then
				error_clr <= '1';
			else
				error_clr <= '0';
			end if;

			-- Shut down write transactions during idle.
			if demo_state = idle then
				wr_active <= '0';
			-- Start a write transaction cycle that exercises the full address range.
			elsif demo_state = write then
				wr_active <= '1';
			end if;
		
			-- Shut down read transactions during idle.
			if demo_state = idle then
				rd_active <= '0';
			-- Start a read transaction cycle that exercises the full address range.
			elsif demo_state = read then
				rd_active <= '1';
			end if;
		
			-- Compare values returned by each read transaction to the expected value and log any errors.
			if demo_state = compare and rd_vld = '1' and rd_data /= expected then
				error_detect <= '1';
			else
				error_detect <= '0';
			end if;
		
	   end if;
	end process;
	
	-- Perform write transactions for one full cycle of the available addresses.
	wr_cmd <= wr_active and wr_rdy and not(wr_add(QDR_ADDR_WIDTH));
	-- Increment the address counter with the start of each write transaction.
	wr_add_ce <= wr_cmd or cnt_clr;
	
	-- Perform read transactions for one full cycle of the available addresses.
	rd_cmd <= rd_active and rd_rdy and not(rd_add(QDR_ADDR_WIDTH));
	-- Increment the address counter with the start of each read transaction.
	rd_add_ce <= rd_cmd or cnt_clr;
		
	-- Increment the expected value counter with the completion of each read transaction.
	rd_expect_ce <= rd_vld or cnt_clr;
	
	-- Increment the error counter with each mismatch between the return value and the expected value.
	error_ce <= error_detect or error_clr;

	-- Assemble a unique write data field with the current address stored in the lower 36 bits.
	wr_data(35 downto 0) <= (35 downto QDR_ADDR_WIDTH => '0') & wr_add(QDR_ADDR_WIDTH-1 downto 0);
	-- Force all of the bits to toggle during the 18-bit DDR cycle that writes the upper 36 bits.
	wr_data(71 downto 36) <= wr_data(35 downto 18) & not(wr_data(35 downto 18));
	
	-- Assemble the expected data field from the current read address.
	expected(35 downto 0) <= (35 downto QDR_ADDR_WIDTH => '0') & rd_expect(QDR_ADDR_WIDTH-1 downto 0);
	expected(71 downto 36) <= expected(35 downto 18) & not(expected(35 downto 18));
	
	-- Generat a busy status flag.
	sram_busy <= '0' when demo_state = idle else '1';
		
--***************************************************** INSTANTIATE SRAM *******************************************************
		
	SRAM : entity work.qdr_sram
	port map (
		rst					=> rst,
		qdriip_clk			=> qdriip_clk,
		clk200				=> clk200,
		dll_on				=> '1',
		wr_clk				=> clk200,
		wr_rdy				=> wr_rdy,
		wr_cmd				=> wr_cmd,
		wr_add				=> wr_add(QDR_ADDR_WIDTH-1 downto 0),
		wr_data				=> wr_data,
		wr_bw 				=> (others => '1'),
		rd_clk				=> clk200,
		rd_rdy				=> rd_rdy,
		rd_cmd				=> rd_cmd,
		rd_add				=> rd_add(QDR_ADDR_WIDTH-1 downto 0),
		rd_data 			=> rd_data,
		rd_vld				=> rd_vld,
		rd_ack 				=> rd_vld,
		dll_rst_start		=> dll_rst_start,
		clk_lck_status		=> clk_lck_status,
		qdriip_cq			=> qdriip_cq,
		qdriip_q			=> qdriip_q,
		qdriip_qvld			=> qdriip_qvld,
		qdriip_k			=> qdriip_k,
		qdriip_d			=> qdriip_d,
		qdriip_sa			=> qdriip_sa,
		qdriip_w_n			=> qdriip_w_n,
		qdriip_r_n			=> qdriip_r_n,
		qdriip_bw_n			=> qdriip_bw_n,
		qdriip_dll_off_n	=> qdriip_dll_off_n
	);

end behavioral;
