--*******************************************************************************************************************************
--
-- Description: This module is used to ensure that the delay-locked loop (DLL) internal to the QDR SRAM is stable before the
--				clock lock function is used to optimize timing at the read data port.
--					
-- Rev History: 08-04-2016:	Changed the name of qdriip_sys_clk to qdriip_clk to distinguish spread spectrum clock name.
-- 				07-14-2015:	Initial release.
--
--*******************************************************************************************************************************

library ieee;
use ieee.std_logic_1164.all;
use IEEE.numeric_std.all; 

Library UNISIM; 
use UNISIM.Vcomponents.all; 

entity qdr_dll is

port (
	-- Reset and clocks supplied by the application logic.
	rst         	: in std_logic;		-- Active high asynchronous reset.       
	clk200 			: in std_logic;		-- 200 MHz clock on global clock buffer.
	qdriip_clk		: in std_logic;		-- QDR II+ fixed frequency or spread spectrum clock.

	-- Signals used to trigger and monitor the DLL reset process.
	dll_rst_start	: in std_logic; 	-- Single period enable initiates the DLL reset process. 
	dll_rst_busy	: out std_logic; 	-- Logic one indicates that the DLL reset process is active. 
	dll_rst_done	: out std_logic; 	-- Single period lock done pulse to indicate that the process is complete. 
		
	-- Connections to the IODELAY function in the IOB that needs to be phase aligned.
	dll_on			: in std_logic;		-- Logic one enables the DLL.
	dll_off_n		: out std_logic		-- Logic zero disables the DLL.
);
end qdr_dll;

architecture behavioral of qdr_dll is

	type QDR_DLL_STATE is (idle, stable, reset, settle, done);	-- Type definitions for the state machine.
	signal dll_state 		: QDR_DLL_STATE;		-- Current state.
	signal timer			: unsigned(9 downto 0);	-- General purpose timer.
	signal dll_start_rclk	: std_logic;	-- Single period start command in the 200 MHz clock domain.
	signal dll_on_rclk		: std_logic;	-- Reclocked copy of dll_on in the 200 MHz clock domain.
	signal dll_rstn			: std_logic;	-- Logic zero resets the DLL.
	signal dll_busy			: std_logic;	-- Logic one indicates that the DLL reset process is active.
	signal dll_done			: std_logic;	-- Single period lock done pulse to indicate that the process is complete.
	
begin

--******************************************* RECLOCK STATE MACHINE CONTROL INPUTS *********************************************

    -- Generate a single period start strobe in the clk200 clock domain.
    -- Note: The incoming dll_rst_start control does not have to originate from the 200 MHz clock domain if it is not strobe.
    DLL_RST_START_XING : entity work.clock_crossing
	port map (
		source_clk			=> clk200,
		target_clk  		=> clk200,
		rst					=> rst,
		source_to_target	=> dll_rst_start,
		target_enable		=> dll_start_rclk
	);
	
    -- Reclock static control inputs.
	process(rst, clk200)
	begin
		if rst = '1' then
			dll_on_rclk <= '0';
		elsif rising_edge(clk200) then
			dll_on_rclk <= dll_on;
		end if;
	end process;
	
--************************************************* STATE MACHINE TRANSITIONS **************************************************

	process(rst, clk200)
	begin
		if rst = '1' then
			dll_state <= idle;
			
		elsif rising_edge(clk200) then
			
			-- The following state machine ensures that the DLL internal to the QDR is stable before clock lock is run.
			case dll_state is		
			
				-- Remain idle while waiting to initiate a dll lock sequence or the dll is disabled from application logic.
				when idle =>
					-- Detect the dll_rst_start pulse to begin the clock lock process.	
					if dll_start_rclk = '1' or dll_on_rclk = '0' then
						dll_state <= stable;
					end if;
					
				-- Wait for the QDR clock to run stable for 30 ns as required by the QDR DLL.
				when stable =>
					if timer = "0000000111" then
						dll_state <= reset;
					end if;
					
				-- Reset the QDR DLL.
				when reset =>
					if dll_on_rclk = '1' then
						dll_state <= settle;
					end if;
					
				-- Allow the QDR DLL to settle (2048 cycles of the QDR clock).
				when settle =>
					if timer = "1111111111" then
						dll_state <= done;
					end if;
						
				-- Generate a single clock period done pulse.
				when done =>
					dll_state <= idle;
					
				-- Define the default state to be null.
				when others  =>
					null;
			end case;
			
		end if;
	end process;
	
--*************************************************** STATE MACHINE EVENTS *****************************************************
			
	process(rst, clk200)
	begin
		if rst = '1' then
			dll_rstn <= '0';
			timer <= (others => '0');
			dll_busy <= '0';
			dll_done <= '0';
			
		elsif rising_edge(clk200) then
			
			-- Reset the QDR DLL.
			if dll_state = reset then
				dll_rstn <= '0';
			else
				dll_rstn <= '1';
			end if;
			
			-- Create a counter that can be used as a timer.
			if dll_state = idle or dll_state = reset then
				timer <= (others => '0');
			else
				timer <= timer + "0000000001";
			end if;
					
			-- Generate a busy flag that can be used to reset the clock lock function.
			if dll_state = idle then
				dll_busy <= '0';
			else
				dll_busy <= '1';
			end if;
			
			-- Generate a busy flag that can be used to reset the clock lock function.
			if dll_state = done then
				dll_done <= '1';
			else
				dll_done <= '0';
			end if;
			
		end if;
	end process;
	
--****************************************** RECLOCK QDR CONTROL AND STATUS OUTPUTS ********************************************

    -- Reclock static control and status signals.
	process(rst, qdriip_clk)
	begin
		if rst = '1' then
			dll_off_n <= '0';
			dll_rst_busy <= '0';
		elsif rising_edge(qdriip_clk) then
			dll_off_n <= dll_rstn;
			dll_rst_busy <= dll_busy;
		end if;
	end process;
	
    -- Generate a single period done strobe in the QDR clock domain.
    DONE_XING : entity work.clock_crossing
	port map (
		source_clk			=> clk200,
		target_clk  		=> qdriip_clk,
		rst					=> rst,
		source_to_target	=> dll_done,
		target_enable		=> dll_rst_done
	);
	
end behavioral;
